@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1991, 1997 Free Software Foundation, Inc.
+** Copyright (C) 1991, 1997-2015,2017 Free Software Foundation, Inc.
 **
 ** This file is part of TACK.
 **
@@ -19,7 +19,7 @@
 ** Boston, MA 02110-1301, USA
 */
 
-/* $Id: tack.h,v 1.23 2009/12/26 21:25:11 tom Exp $ */
+/* $Id: tack.h,v 1.64 2017/07/26 08:08:21 tom Exp $ */
 
 #ifndef NCURSES_TACK_H_incl
 #define NCURSES_TACK_H_incl 1
@@ -27,17 +27,45 @@
 /* terminfo action checker include file */
 
 #define MAJOR_VERSION 1
-#define MINOR_VERSION 7
+#define MINOR_VERSION 8
+#define PATCH_VERSION 20170726
 
 #ifdef HAVE_CONFIG_H
 #include <ncurses_cfg.h>
 #else
-#define RETSIGTYPE void
-#define GCC_UNUSED /*nothing*/
+#define GCC_UNUSED		/*nothing */
+#define GCC_NORETURN		/*nothing */
 #define HAVE_GETTIMEOFDAY 1
 #define HAVE_SELECT 1
 #define HAVE_SYS_TIME_H 1
-#define HAVE_SYS_TIME_SELECT 1
+#endif
+
+#ifndef BROKEN_LINKER
+#define BROKEN_LINKER 0
+#endif
+
+#ifndef HAVE_LONG_FILE_NAMES
+#define HAVE_LONG_FILE_NAMES 0
+#endif
+
+#ifndef NCURSES_CONST
+#define NCURSES_CONST const
+#endif
+
+#ifndef NO_LEAKS
+#define NO_LEAKS 0
+#endif
+
+#ifndef USE_DATABASE
+#define USE_DATABASE 0
+#endif
+
+#ifndef USE_TERMCAP
+#define USE_TERMCAP 0
+#endif
+
+#ifndef USE_RCS_IDS
+#define USE_RCS_IDS 0
 #endif
 
 #include <sys/types.h>
@@ -45,39 +73,112 @@
 #include <unistd.h>
 #include <ctype.h>
 #include <string.h>
-#include <signal.h>	/* include before curses.h to work around glibc bug */
+#include <signal.h>		/* include before curses.h to work around glibc bug */
 
 #include <curses.h>
+
+#ifdef NCURSES_VERSION
 #include <term_entry.h>
-#include <nc_tparm.h>
+#else
+#include <term.h>
+#include <termios.h>
+#define TTY struct termios
+#define TERMIOS 1
+#define GET_TTY(fd, buf) tcgetattr(fd, buf)
+#endif
 
 #if USE_RCS_IDS
 #define MODULE_ID(id) static const char Ident[] = id;
 #else
-#define MODULE_ID(id) /*nothing*/
+#define MODULE_ID(id)		/*nothing */
 #endif
 
-#if !HAVE_STRSTR
-extern char *_nc_strstr(const char *, const char *);
-#define strstr(h,n) _nc_strstr(h,n)
+#if defined(__GNUC__) && defined(_FORTIFY_SOURCE)
+#define IGNORE_RC(func) ignore_unused = (int) func
+#else
+#define IGNORE_RC(func) (void) func
+#endif /* gcc workarounds */
+
+#ifndef down_half_line
+#define down_half_line 0	/* NetBSD bug */
+#endif
+#ifndef hue_lightness_saturation
+#define hue_lightness_saturation 0	/* NetBSD bug */
+#endif
+#ifndef plab_norm
+#define plab_norm 0		/* NetBSD bug */
+#endif
+#ifndef prtr_off
+#define prtr_off 0		/* NetBSD bug */
+#endif
+#ifndef prtr_on
+#define prtr_on 0		/* NetBSD bug */
 #endif
 
-#if NO_LEAKS && defined(HAVE__NC_FREE_TIC)
-extern void _nc_free_tic(int);
+#if NO_LEAKS
 extern void tack_edit_leaks(void);
 extern void tack_fun_leaks(void);
-extern void ExitProgram(int);
+#ifdef HAVE__NC_FREE_TINFO
+extern void _nc_free_tinfo(int) GCC_NORETURN;
+#endif
+#ifndef ExitProgram
+extern void ExitProgram(int) GCC_NORETURN;
+#endif
 #else
 #define ExitProgram(code) exit(code)
 #undef  NO_LEAKS
 #define NO_LEAKS 0
 #endif
 
+#ifdef NCURSES_VERSION
+#define ChangeTermInfo(name,value) name = value
+#else
+#define ChangeTermInfo(name,value)	/* nothing */
+#endif
+
 #define FreeIfNeeded(p) if (p) { free(p); p = 0; }
 
-#define CUR_TP      (&(cur_term->type))
+#include <tackgen.h>
+
+#ifdef NCURSES_VERSION
+#define CUR_TP      ((TERMTYPE *)(cur_term))
+#define MAX_BOOLEAN BOOLCOUNT	/* NUM_BOOLEANS(CUR_TP) */
+#define MAX_NUMBERS NUMCOUNT	/* NUM_NUMBERS(CUR_TP) */
 #define MAX_STRINGS NUM_STRINGS(CUR_TP)
 #define STR_NAME(n) ExtStrname(CUR_TP,n,strnames)
+#elif defined(BOOLCOUNT)
+#define MAX_BOOLEAN BOOLCOUNT
+#define MAX_NUMBERS NUMCOUNT
+#define MAX_STRINGS STRCOUNT
+#define STR_NAME(n) strnames[n]
+#else
+#define MAX_BOOLEAN max_booleans
+#define MAX_NUMBERS max_numbers
+#define MAX_STRINGS max_strings
+#define STR_NAME(n) strnames[n]
+#endif
+
+typedef enum {
+    BOOLEAN,
+    NUMBER,
+    STRING
+} NAME_TYPE;
+
+/* see ncurses' nc_tparm.h */
+#define TPARM_ARG long
+#define TPARM_N(n) (TPARM_ARG)(n)
+
+#define TPARM_9(a,b,c,d,e,f,g,h,i,j) tparm(a,TPARM_N(b),TPARM_N(c),TPARM_N(d),TPARM_N(e),TPARM_N(f),TPARM_N(g),TPARM_N(h),TPARM_N(i),TPARM_N(j))
+#define TPARM_8(a,b,c,d,e,f,g,h,i) TPARM_9(a,b,c,d,e,f,g,h,i,0)
+#define TPARM_7(a,b,c,d,e,f,g,h) TPARM_8(a,b,c,d,e,f,g,h,0)
+#define TPARM_6(a,b,c,d,e,f,g) TPARM_7(a,b,c,d,e,f,g,0)
+#define TPARM_5(a,b,c,d,e,f) TPARM_6(a,b,c,d,e,f,0)
+#define TPARM_4(a,b,c,d,e) TPARM_5(a,b,c,d,e,0)
+#define TPARM_3(a,b,c,d) TPARM_4(a,b,c,d,0)
+#define TPARM_2(a,b,c) TPARM_3(a,b,c,0)
+#define TPARM_1(a,b) TPARM_2(a,b,0)
+#define TPARM_1(a,b) TPARM_2(a,b,0)
+#define TPARM_0(a) TPARM_1(a,0)
 
 #define UChar(c)    ((unsigned char)(c))
 
@@ -85,9 +186,13 @@ extern FILE *log_fp;
 extern FILE *debug_fp;
 extern int debug_level;
 extern char temp[];
-extern char tty_basename[];
+extern char *tty_basename;
 extern char tty_shortname[];
 
+#if defined(__GNUC__) && defined(_FORTIFY_SOURCE)
+extern int ignore_unused;
+#endif
+
 #define SYNC_FAILED	0
 #define SYNC_TESTED	1
 #define SYNC_NOT_TESTED	2
@@ -96,9 +201,9 @@ extern char tty_shortname[];
 extern int tty_can_sync;
 extern int total_pads_sent;	/* count pad characters sent */
 extern int total_caps_sent;	/* count caps sent */
-extern int total_printing_characters; /* count printing characters sent */
-extern SIG_ATOMIC_T no_alarm_event; /* TRUE if the alarm has not gone off yet */
-extern unsigned long usec_run_time; /* length of last test in microseconds */
+extern int total_printing_characters;	/* count printing characters sent */
+extern SIG_ATOMIC_T no_alarm_event;	/* TRUE if the alarm has not gone off yet */
+extern unsigned long usec_run_time;	/* length of last test in microseconds */
 extern int raw_characters_sent;	/* Total output characters */
 
 /* Stopwatch event timers */
@@ -113,7 +218,7 @@ extern int raw_characters_sent;	/* Total output characters */
 #define SLOW_TERMINAL_EXIT if (!test_complete && !EXIT_CONDITION) { break; }
 #define CAP_NOT_FOUND if (auto_pad_mode) return
 
-extern char letters[26+1];
+extern char letters[26 + 1];
 #define NEXT_LETTER letter = letters[letter_number =\
 	letters[letter_number + 1] ? letter_number + 1 : 0]
 
@@ -129,9 +234,9 @@ extern int char_count, line_count, expand_chars;
 extern int can_go_home, can_clear_screen;
 
 extern int translate_mode, scan_mode;
-extern int auto_pad_mode;		/* TRUE for auto time tests */
+extern int auto_pad_mode;	/* TRUE for auto time tests */
 extern int char_mask;
-extern int hex_out;			/* Display output in hex */
+extern int hex_out;		/* Display output in hex */
 
 /* Parity bit macros */
 #define STRIP_PARITY 0x7f
@@ -165,22 +270,22 @@ extern int send_reset_init;
 #define MAX_SCAN 256
 
 /* translate mode default strings */
-struct default_string_list {
-	const char *name;	/* terminfo name */
-	const char *value;	/* value of default string */
-	int index;		/* index into the strfname[] array */
-};
+typedef struct default_string_list {
+    const char *name;		/* terminfo name */
+    const char *value;		/* value of default string */
+    int index;			/* index into the strfname[] array */
+} DefaultStringList;
 
 #define TM_last 8
 extern struct default_string_list TM_string[TM_last];
 
 /* attribute structure definition */
-struct mode_list {
-	const char *name;
-	const char *begin_mode;
-	const char *end_mode;
-	int number;
-};
+typedef struct mode_list {
+    const char *name;
+    const char *begin_mode;
+    const char *end_mode;
+    int number;
+} ModeList;
 
 extern const struct mode_list alt_modes[];
 extern const int mode_map[];
@@ -199,14 +304,14 @@ extern const int mode_map[];
 
 extern int tt_delay_max;	/* max number of milliseconds we can delay */
 extern int tt_delay_used;	/* number of milliseconds consumed in delay */
-extern const char *tt_cap[TT_MAX]; /* value of string */
+extern const char *tt_cap[TT_MAX];	/* value of string */
 extern int tt_affected[TT_MAX];	/* lines or columns effected (repetition
 				   factor) */
 extern int tt_count[TT_MAX];	/* Number of times sent */
 extern int tt_delay[TT_MAX];	/* Number of milliseconds delay */
 extern int ttp;			/* number of entries used */
 
-extern const char *tx_cap[TT_MAX]; /* value of string */
+extern const char *tx_cap[TT_MAX];	/* value of string */
 extern int tx_affected[TT_MAX];	/* lines or columns effected (repetition
 				   factor) */
 extern int tx_count[TT_MAX];	/* Number of times sent */
@@ -220,39 +325,42 @@ extern unsigned long tx_cps;	/* characters per second */
 	Menu control for tack.
 */
 
-struct test_results {
-	struct test_results *next;	/* point to next entry */
-	struct test_list *test;	/* Test which got these results */
-	int reps;		/* repeat count */
-	int delay;		/* delay times 10 */
-};
+struct test_list;
+typedef void (TestFunc) (struct test_list * t, int *state, int *ch);
 
-struct test_list {
-	int flags;		/* Test description flags */
-	int lines_needed;	/* Lines needed for test (0->no action) */
-	const char *caps_done;	/* Caps shown in Done message */
-	const char *caps_tested; /* Other caps also being tested */
-	const char *menu_entry;	/* Menu entry text (optional) */
-				/* Function that does testing */
-	void (*test_procedure)(struct test_list *, int *, int *);
-	struct test_menu *sub_menu;	/* Nested sub-menu */
-};
+struct test_menu;
+typedef void (MenuFunc) (struct test_menu *);
 
-struct test_menu {
-	int flags;		/* Menu feature flag */
-	int default_action;	/* Default command if <cr> <lf> entered */
-	const char *menu_text;	/* Describe this test_menu */
-	const char *menu_title;	/* Title for the menu */
-	const char *ident;	/* short menu name */
-	const char *standard_tests;	/* Standard test text */
-				/* print current settings (optional) */
-	void (*menu_function)(struct test_menu *);
-	struct test_list *tests;	/* Pointer to the menu/function pairs */
-	struct test_list *resume_tests;	/* Standard test resume point */
-	int resume_state;	/* resume state of test group */
-	int resume_char;	/* resume ch of test group */
-};
+typedef struct test_results {
+    struct test_results *next;	/* point to next entry */
+    struct test_list *test;	/* Test which got these results */
+    int reps;			/* repeat count */
+    int delay;			/* delay times 10 */
+} TestResults;
 
+typedef struct test_list {
+    int flags;			/* Test description flags */
+    int lines_needed;		/* Lines needed for test (0->no action) */
+    const char *caps_done;	/* Caps shown in Done message */
+    const char *caps_tested;	/* Other caps also being tested */
+    const char *menu_entry;	/* Menu entry text (optional) */
+    TestFunc *test_procedure;	/* Function that does testing */
+    struct test_menu *sub_menu;	/* Nested sub-menu */
+} TestList;
+
+typedef struct test_menu {
+    int flags;			/* Menu feature flag */
+    int default_action;		/* Default command if <cr> <lf> entered */
+    const char *menu_text;	/* Describe this test_menu */
+    const char *menu_title;	/* Title for the menu */
+    const char *ident;		/* short menu name */
+    const char *standard_tests;	/* Standard test text */
+    MenuFunc *menu_function;	/* print current settings (optional) */
+    TestList *tests;		/* Pointer to the menu/function pairs */
+    TestList *resume_tests;	/* Standard test resume point */
+    int resume_state;		/* resume state of test group */
+    int resume_char;		/* resume ch of test group */
+} TestMenu;
 
 /* menu flags */
 #define MENU_100c	0x00001a00	/* Augment 100% of columns */
@@ -289,7 +397,18 @@ struct test_menu {
 #define REQUEST_PROMPT 256
 
 /* tack.c */
-extern struct test_menu edit_menu;
+#ifdef NCURSES_VERSION
+extern TestMenu edit_menu;
+#define MY_EDIT_MENU	{0, 0, 0, 0, "e) edit terminfo", 0, &edit_menu},
+#else
+#define MY_EDIT_MENU		/* nothing */
+#endif
+#ifdef DEBUG
+#define TACKMSG(p) p
+extern void TackMsg(const char *,...) GCC_PRINTFLIKE(1,2);
+#else
+#define TACKMSG(p)		/* nothing */
+#endif
 extern void show_usage(char *);
 extern void print_version(void);
 
@@ -301,7 +420,6 @@ extern char *hex_expand_to(char *, int);
 extern char *print_expand(char *);
 extern int getchp(int);
 extern int getnext(int);
-extern int tc_putch(int);
 extern int tc_putp(const char *);
 extern int wait_here(void);
 extern void go_home(void);
@@ -321,71 +439,84 @@ extern void put_str(const char *);
 extern void put_this(int);
 extern void putchp(int);
 extern void putln(const char *);
-extern void read_string(char *, int);
+extern void read_string(char *, size_t);
 extern void three_digit(char *, int);
 extern void tt_putp(const char *);
 extern void tt_putparm(NCURSES_CONST char *, int, int, int);
 extern void tt_tputs(const char *, int);
 
+/* Solaris is out of step - humor it */
+#if defined(__EXTENSIONS__) && !defined(NCURSES_VERSION)
+#define TC_PUTCH char
+#else
+#define TC_PUTCH int
+#endif
+extern int tc_putch(TC_PUTCH);
+
 #define put_that(n) put_this((int) (n))
 
 /* control.c */
-extern struct test_list color_test_list[];
-extern char *liberated(char *);
+extern TestList color_test_list[];
+extern char *liberated(const char *);
 extern char txt_longer_augment[80];
 extern char txt_longer_test_time[80];
 extern char txt_shorter_augment[80];
 extern char txt_shorter_test_time[80];
 extern int msec_cost(const char *const, int);
-extern int skip_pad_test(struct test_list *, int *, int *, const char *);
+extern int skip_pad_test(TestList *, int *, int *, const char *);
 extern int sliding_scale(int, int, unsigned long);
 extern int still_testing(void);
 extern long event_time(int);
 extern void control_init(void);
-extern void dump_test_stats(struct test_list *, int *, int *);
+extern void dump_test_stats(TestList *, int *, int *);
 extern void event_start(int);
-extern void longer_augment(struct test_list *, int *, int *);
-extern void longer_test_time(struct test_list *, int *, int *);
-extern void pad_test_shutdown(struct test_list *, int);
+extern void longer_augment(TestList *, int *, int *);
+extern void longer_test_time(TestList *, int *, int *);
+extern void pad_test_shutdown(TestList *, int);
 extern void pad_test_startup(int);
 extern void page_loop(void);
 extern void set_augment_txt(void);
-extern void shorter_augment(struct test_list *, int *, int *);
-extern void shorter_test_time(struct test_list *, int *, int *);
+extern void shorter_augment(TestList *, int *, int *);
+extern void shorter_test_time(TestList *, int *, int *);
 
 /* charset.c */
-extern struct test_list acs_test_list[];
+extern TestList acs_test_list[];
 extern void set_attr(int);
 extern void eat_cookie(void);
-extern void put_mode(char *);
+extern void put_mode(const char *);
 
 /* crum.c */
-extern struct test_list crum_test_list[];
+extern TestList crum_test_list[];
 
 /* ansi.c */
-extern void tools_status(struct test_list *, int *, int *);
-extern void tools_charset(struct test_list *, int *, int *);
-extern void tools_sgr(struct test_list *, int *, int *);
+extern void tools_status(TestList *, int *, int *);
+extern void tools_charset(TestList *, int *, int *);
+extern void tools_sgr(TestList *, int *, int *);
 
 /* edit.c */
-extern struct test_menu change_pad_menu;
-extern struct test_list edit_test_list[];
-extern char *get_string_cap_byname(const char *, const char **);
+#ifdef NCURSES_VERSION
+extern TestMenu change_pad_menu;
+extern TestList edit_test_list[];
+#define MY_PADS_MENU	{0, 0, 0, 0, "p) change padding", 0, &change_pad_menu},
+#else
+#define MY_PADS_MENU		/* nothing */
+#endif
+extern const char *get_string_cap_byname(const char *, const char **);
 extern int cap_match(const char *names, const char *cap);
 extern int get_string_cap_byvalue(const char *);
 extern int user_modified(void);
 extern void can_test(const char *, int);
 extern void cap_index(const char *, int *);
 extern void edit_init(void);
-extern void save_info(struct test_list *, int *, int *);
-extern void show_report(struct test_list *, int *, int *);
+extern void save_info(TestList *, int *, int *);
+extern void show_report(TestList *, int *, int *);
 
 /* fun.c */
-extern struct test_list funkey_test_list[];
-extern struct test_list printer_test_list[];
+extern TestList funkey_test_list[];
+extern TestList printer_test_list[];
 extern void enter_key(const char *, char *, char *);
 extern int tty_meta_prep(void);
-extern void tools_report(struct test_list *, int *, int *);
+extern void tools_report(TestList *, int *, int *);
 
 /* init.c */
 extern void reset_init(void);
@@ -398,15 +529,15 @@ extern void bye_kids(int);
 /* scan.c */
 extern char **scan_up, **scan_down, **scan_name;
 extern int scan_key(void);
-extern unsigned scan_max;	/* length of longest scan code */
-extern unsigned *scan_tested, *scan_length;
+extern size_t scan_max;		/* length of longest scan code */
+extern size_t *scan_tested, *scan_length;
 extern void scan_init(char *fn);
 
 /* sysdep.c */
 extern int initial_stty_query(int);
 extern int stty_query(int);
 extern void ignoresig(void);
-extern void read_key(char *, int);
+extern void read_key(char *, size_t);
 extern void set_alarm_clock(int);
 extern void spin_flush(void);
 extern void tty_init(void);
@@ -416,28 +547,28 @@ extern void tty_set(void);
 
 /* menu.c */
 extern char prompt_string[80];	/* menu prompt storage */
-extern int subtest_menu(struct test_list *, int *, int *);
-extern struct test_list *augment_test;
-extern void generic_done_message(struct test_list *, int *, int *);
-extern void menu_can_scan(const struct test_menu *);
-extern void menu_clear_screen(struct test_list *, int *, int *);
-extern void menu_display(struct test_menu *, int *);
+extern int subtest_menu(TestList *, int *, int *);
+extern TestList *augment_test;
+extern void generic_done_message(TestList *, int *, int *);
+extern void menu_can_scan(const TestMenu *);
+extern void menu_clear_screen(TestList *, int *, int *);
+extern void menu_display(TestMenu *, int *);
 extern void menu_prompt(void);
-extern void menu_reset_init(struct test_list *, int *, int *);
-extern void pad_done_message(struct test_list *, int *, int *);
+extern void menu_reset_init(TestList *, int *, int *);
+extern void pad_done_message(TestList *, int *, int *);
 
 /* modes.c */
-extern struct test_list mode_test_list[];
+extern TestList mode_test_list[];
 
 /* pad.c */
-extern struct test_list pad_test_list[];
+extern TestList pad_test_list[];
 
 /* sync.c */
-extern struct test_menu sync_menu;
+extern TestMenu sync_menu;
 extern int tty_sync_error(void);
 extern void flush_input(void);
-extern void sync_handshake(struct test_list *, int *, int *);
-extern void sync_test(struct test_menu *);
+extern void sync_handshake(TestList *, int *, int *);
+extern void sync_test(TestMenu *);
 extern void verify_time(void);
 
 #endif /* NCURSES_TACK_H_incl */
